from base64 import b64encode
import datetime
import asyncio
import hashlib
import struct
import hmac
import json

from .errors import NuVotifierHeaderError, NuVotifierResponseError, ClientNotConnectedError


class NuVotifierClient:
    def __init__(self, host: str, port: int, token: str):
        self.host = host
        self.port = port
        self.token = token

        self._ready = False

        self._reader = None
        self._writer = None

    async def __aenter__(self):
        await self.connect()
        return self

    async def __aexit__(self, *args, **kwargs):
        await self.close()

    async def connect(self) -> "NuVotifierClient":
        if self._ready:
            return

        self._reader, self._writer = await asyncio.open_connection(self.host, self.port)

    async def close(self) -> None:
        if self._ready:
            self._writer.close()
            await self._writer.wait_closed()

            self._reader = None
            self._writer = None

            self._ready = False

    async def vote(self, username: str) -> None:
        if not self._ready:
            raise ClientNotConnectedError

        header = await self._reader.read(64)

        if not header:
            raise NuVotifierHeaderError(header)

        header_split = header.split()

        if len(header_split) != 3:
            raise NuVotifierHeaderError(header)

        # create packet data
        payload = json.dumps(
            {
                "username": username,
                "serviceName": "minecraft.global",
                "timestamp": round(datetime.datetime.utcnow().timestamp() * 1000),
                "address": f"{self.host}:{self.port}",
                "challenge": header_split[2].decode(),
            }
        )
        signature = b64encode(hmac.digest(self.token.encode(), payload.encode(), hashlib.sha256)).decode()
        message = json.dumps({"signature": signature, "payload": payload}).encode()

        # write packet
        self._writer.write(struct.pack(">HH", 0x733A, len(message)) + message)
        await self._writer.drain()

        response = json.loads(await self._reader.read(256))

        if response["status"] != "ok":
            raise NuVotifierResponseError(response)



class VotifierError(Exception):
    """Base class for votifier exceptions"""


class ClientNotConnectedError(VotifierError):
    """Raised when a client is used without being connected first"""


class NuVotifierError(VotifierError):
    """Base class for NuVotifier exceptions"""


class NuVotifierHeaderError(NuVotifierError):
    def __init__(self, header: bytes):
        super().__init__("Invalid header received from votifier server.")
        self.header = header


class NuVotifierResponseError(NuVotifierError):
    def __init__(self, response: dict):
        super().__init__("Response from votifier server was not OK.")
        self.response = response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiovotifier']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'aiovotifier',
    'version': '0.1.3',
    'description': 'An asynchronous MInecraft server votifier client in Python',
    'long_description': '# Aio-Votifier ![Code Quality](https://www.codefactor.io/repository/github/iapetus-11/aio-votifier/badge) ![PYPI Version](https://img.shields.io/pypi/v/aiovotifier.svg) ![PYPI Downloads](https://img.shields.io/pypi/dw/aiovotifier?color=0FAE6E) ![Views](https://api.ghprofile.me/view?username=iapetus-11.aio-votifier&color=0FAE6E&label=views&style=flat)\n*An asynchronous MInecraft server votifier client in Python*\n\n## Example Usage:\n```py\nfrom aiovotifier import NuVotifierClient\nimport asyncio\n\nasync def main():\n    async with NuVotifierClient("127.0.0.1", 8192, "token") as client:\n        await client.vote("Iapetus11")\n\nasyncio.run(main())\n```\nor\n```py\nfrom aiovotifier import NuVotifierClient\nimport asyncio\n\nasync def main():\n    client = NuVotifierClient("127.0.0.1", 8192, "token")\n    await client.connect()\n\n    await client.vote("Iapetus11")\n\n    await client.close()\n\nasyncio.run(main())\n```\n',
    'author': 'Milo Weinberg',
    'author_email': 'iapetus011@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
